%AliceLocalizer.m
%


%DESCRIPTION
% A block design experiment that plays Alice in Wonderland passages read
% by a native speaker. Conditions are 'intact' and 'degraded'. You can
% optionally include a foreign language condition as well.
%
%
%TIMING - WITHOUT FOREIGN CONDITION
% Trial blocks: 12 (18 seconds each)
% Fixation blocks: 4 (12 seconds each)
% Total runtime: 264 seconds (4:24)
% IPS for TR=2: 132
%
%TIMING - WITH FOREIGN CONDITION
% Trial blocks: 12 (18 seconds each)
% Fixation blocks: 3 (12 seconds each)
% Total runtime: 252 seconds (4:12)
% IPS for TR=2: 126
%
%
%HOW TO RUN
% 1) Make sure the audio files are set up correctly. See AUDIO FILE SETUP
%    below if you're having problems with the audio files.
%
% 2) Call this function from the MATLAB command line. The inputs
%    'subjectID', 'mainLanguage', 'foreignLanguage', and 'list' must be the
%    same across runs for a subject in a given session.
%
%
%FUNCTION CALL
% AliceLocalizer(subjectID, mainLanguage, foreignLanguage, list, run, conditionOrder, randSeed)
%
%   - subjectID
%       type: string (any)
%       A unique subject ID. Must be the same across runs for a subject.
%
%   - mainLanguage
%       type: string (any language that is set up in the stimuli directory)
%       The primary language you want to test. This language will be
%       presented in both 'intact' and 'degraded' versions.
%
%   - foreignLangauge
%       type: string (any language that is set up in the stimuli directory)
%       Langauge you want to use for the foreign condition. If you do not
%       want to include the foreign condition, set this equal to ''
%
%   - list
%       type: integer (1 or 2)
%       Specifies the subset of materials to use. This input (along with
%       subjectID, mainLanguage, and foreignLangauge) must be the same
%       across runs for a subject.
%
%   - run
%       type: integer (1 or 2 if not using the foreign language condition, 
%                      1, 2, or 3 if using the foreign language condition)
%       The run number. You must do run 1 first.
%
%   - conditionOrder
%       type: integer (1 or 2 if not using the foreign language condition, 
%                      1, 2, or 3 if using the foreign language condition)
%       Specifies the condition block ordering.
%
%   - randSeed
%       type: integer (any positive integer)
%       Seeds the random number generator. It's only required on the first
%       run. If you use the same randSeed, list, and conditionOrder for two
%       subjects, the same materials will be presented in the same order
%       for those subjects.
%
%
%EXAMPLE FUNCTION CALL
% AliceLocalizer('subj01', 'English', '', 2, 1, 1, 937)
% AliceLocalizer('subj01', 'English', '', 2, 2, 2, 937)
%
%
%OUTPUTS
% Data files indicating which item was used on each trial are stored in:
%   (if using the foreign language condition) data/3conditions/
%   (if  not using the foreign language condition) data/2conditions/
%
%
%AUDIO FILE SETUP
% 1) Make sure there is a directory called 'stimuli' in the same location
%    as the 'AliceLocalizer.m' script.
%
% 2) In the 'stimuli' directory, make sure there is one folder for each
%    language you want to use (ex: 'stimuli/English/')
%
% 3) A given langauge folder should have 2 sub-folders called 'intact' and
%   'degraded'. The actual audio files should be in these directories.
%
% 4) The intact audio files should be named '<num>_intact.wav' and the
%    degraded files should be named '<num>_degraded.wav' where num = 1 - 24
%    (ex: 'stimuli/English/intact/3_intact.wav')
%
% Notes
%   * This script automatically trims the duration of the audio files to
%     duration.audio (specified below). All of the audio files in the
%     stimuli directory should be at least this length.
%
%   * The script automatically fades out the volume at the end of each
%     audio clip, so the .wav files should not have this effect built-in.
%     If you want to tinker with how the fade out feature works, check out
%     private/setUpAudioFiles.m
%
%   * This script tries to normalize the intensity of each sound file so
%     that they all play at a consistent volume. It also makes sure that
%     the same sound data is played through the left and right channels.
%     Still, you should test the script with any new sound files before
%     running it in the scanner.
%
%
%CHANGE LOG
% 2016-05-10: created (Zach Mineroff - mineroff@mit.edu)
%
%

function AliceLocalizer(subjectID, mainLanguage, foreignLanguage, list, run, conditionOrder, randSeed)
    %% Experiment parameters
    %Timing (in seconds)
    duration.fix   = 12; %Length of a fixation block
    duration.audio = 18; %Length of an audio block; All audio files should be at least this long
    duration.fade  = 2;  %How long before the END of an audio block to start fading out the volume
    
    %Display options
    screenColor = [255, 255, 255]; %RGB array; Use [255 255 255] for white
    fontSize = 40;
    
    
    %% Inputs and output file
    %Store the inputs as a struct and make sure they're valid
    if nargin < 7
        randSeed = [];
    end
    inputs.subjectID = subjectID;
    inputs.mainLanguage = mainLanguage;
    inputs.foreignLanguage = foreignLanguage;
    inputs.list = list;
    inputs.run = run;
    inputs.conditionOrder = conditionOrder;
    inputs.randSeed = randSeed;
    
    validateInputs(inputs);
    
    %Determine whether or not to include the foreign language condition
    usingForeignLanguage = ~(isempty(foreignLanguage) || ...
                             all(isspace(foreignLanguage)));
	
	inputs.usingForeignLanguage = usingForeignLanguage;
    
    %Get a string of the filename we want to save
	dataFileToSave = getDataFilename(inputs);
    
    %Seed the random number generator
    oldRandSettings = rng;
    if run == 1
        rng('default');
        rng(randSeed);
    end
    
    
    %% Read in materials
	if usingForeignLanguage
        materialsDir = fullfile(pwd, 'materials', '3conditions');
        materialsFilename = sprintf('%s_%s_%s_list%d.mat', ...
                                    subjectID, mainLanguage, ...
                                    foreignLanguage, list);
	else
        materialsDir = fullfile(pwd, 'materials', '2conditions');
        materialsFilename = sprintf('%s_%s_list%d.mat', ...
                                     subjectID, mainLanguage, list);
	end
    
	materialsFile = fullfile(materialsDir, materialsFilename);
    
    %Set up the materials for all runs on the first run
	if run == 1
        if ~exist(materialsDir, 'dir')
            mkdir(materialsDir);
        end
        createSubjMaterials(inputs, materialsFile);
	end
    try
        load(materialsFile)
    catch
        error(['\nCould not load materials file ''%s''\n\n'...
               'Please make sure that run is 1 for the first run and 2 for the second.\n\n'...
               'Also make sure that the following inputs are the SAME for run 1 and run 2:\n'...
               '\t1) subjectID\n'...
               '\t2) mainLanguage\n'...
               '\t3) foreignLanguage\n'...
               '\t4) list'],...
               materialsFile);
    end
    
    runField = sprintf('run%d', run);
    materials = subjMaterials.(runField);
    
    
    %% Trial information
    fprintf('Setting up run...\n\n');
    
    %Set up condition order
	conditionBlocks = setBlockOrder(inputs);
    materials = putMaterialsInOrder(materials, conditionBlocks);
    
    %Get trial start times
    trialOnsets = trialOnsetTimes(conditionBlocks, duration);
    
    %Set up the data table that we want to save
    results = createResultsTable(inputs, materials, conditionBlocks);
    
    %Make sure the dimensions of the trial info arrays all agree
    trialArrayDims = [length(conditionBlocks);
                      length(materials);
                      length(trialOnsets);
                      height(results)];
	
    trialDimsAgree = all(trialArrayDims == trialArrayDims(1));
	
    if ~trialDimsAgree
        error(['The dimensions of conditionBlocks, materials, results, '...
               'and trialOnsets do not agree.'])
    end
    
    numTrials = length(conditionBlocks); %including fixation
    
    isFixBlock = strcmp(conditionBlocks, 'fix');
    
    
	%% Set up audio files
    fprintf('Loading audio files...\n\n');
	[pahandle, buffers] = setUpAudioFiles(materials, duration);
    
    %Time it takes to fill the audio buffer (seconds)
    fillBufferTime = 0.05;
    
    %Seconds before starting/stoping audio that you can escape the experiment
    audioQueueTime = 0.5;
    
    assert(audioQueueTime > fillBufferTime, ['audioQueueTime must be ' ...
                                             'greater than fillBufferTime']);
    
    %Fill buffer with first audio file
    soundNum = 1;
    PsychPortAudio('FillBuffer', pahandle, buffers(soundNum));
    
    
    %% Set up screen and keyboard for Psychtoolbox
    %Screen
    [~, wPtr, oldScreenPrefs] = openPTBscreen(screenColor);
    Screen('TextSize', wPtr, fontSize);
    
    %Keyboard
    [~, Button, kbIdx, oldKeyboardPrefs] = setUpPTBkeyboard();
    
    
    %% Present the run
	fprintf('\nStarting run %d\n\n', run);
	try
        %Enable high-priority mode
        topPriorityLevel = MaxPriority(wPtr);
        Priority(topPriorityLevel);
        
        %% Wait for trigger
        trialNum = 0;
        waitingText = 'Waiting for scanner...';
        DrawFormattedText(wPtr, waitingText, 'center', 'center');
        Screen('Flip', wPtr);
        
        %Draw fixation cross
        DrawFormattedText(wPtr, '+', 'center', 'center');
        Screen('DrawingFinished', wPtr);
        
        %Flip screen when scanner triggers
        runOnset = waitForTrigger(kbIdx, Button.trigger, Button.escape, wPtr);
        
        trialOnsets = trialOnsets + runOnset;
        
        
        %% Present each trial
        for trialNum = 1:numTrials
            trialOnset = trialOnsets(trialNum);
            
            %Fixation
            if isFixBlock(trialNum)
                results.onset{trialNum} = trialOnset - runOnset;
                
                lastEscapeTime = trialOnset + duration.fix - audioQueueTime;
                waitUntilTime(kbIdx, Button.escape, lastEscapeTime);
                continue
            end
            
            %Start audio
            actualAudioStart = PsychPortAudio('Start', pahandle, [], trialOnset, 1);
            
            %End a little early so we can load & start the next one on time
            audioEndTime = trialOnset + duration.audio - fillBufferTime;
            
            results.onset{trialNum} = actualAudioStart - runOnset;
            
            lastEscapeTime = trialOnset + duration.audio - audioQueueTime;
            waitUntilTime(kbIdx, Button.escape, lastEscapeTime);
            
            PsychPortAudio('Stop', pahandle, 1, [], [], audioEndTime);
            
            %Buffer the next audio file
            soundNum = soundNum + 1;
            if soundNum <= length(buffers)
                PsychPortAudio('FillBuffer', pahandle, buffers(soundNum));
            end
        end
        
	catch errorInfo
        fprintf('\n\n%s%s\n\n', 'error message: ', errorInfo.message)
        for k=1:length(errorInfo.stack)
            disp(errorInfo.stack(k))
        end
        
        if trialNum == 0
            fprintf('\n%s\n\n', 'Run stopped while waiting for trigger');
        else
            fprintf('\n%s%d\n\n', 'Run stopped during trial ', trialNum);
        end
	end
	
    
    %% Done
	%Disable high-priority mode
    Priority(0);
    
    %Save all behavioral data to csv
	writetable(results, dataFileToSave);
    
    %Close the PTB screen and audio port
	Screen('CloseAll');
    ShowCursor;
    
    PsychPortAudio('DeleteBuffer', [], 1); %delete all buffers
    PsychPortAudio('Close');
    
	%Restore the old level.
    Screen('Preference', 'VisualDebugLevel',    oldScreenPrefs.VisualDebugLevel);
    Screen('Preference', 'SuppressAllWarnings', oldScreenPrefs.SuppressAllWarnings);
    Screen('Preference', 'TextRenderer',        oldScreenPrefs.TextRenderer);
    Screen('Preference', 'SkipSyncTests',       oldScreenPrefs.SkipSyncTests);
    
    RestrictKeysForKbCheck(oldKeyboardPrefs.enableKeys);
    
    rng(oldRandSettings);
end
