function createSubjMaterials( inputs, materialsFile )
%createSubjMaterials
%   Creates a structure called subjMaterials that indicates which item to
%   use for each condition on each run.
%
%   Examples:
%       subjMaterials.run1.intact(1) might be 'English/intact/4_intact.wav'
%       subjMaterials.run1.degraded(5) might be 'English/degraded/7_degraded.wav'
%       subjMaterials.run2.foreign(3) might be 'French/intact/2_intact.wav'
%
%   This structure is saved as a mat file called <materialsFile>
%
%   You should call this function on the first run of the script ONLY.
%   On all subsequent runs, you should just load the relevant mat file.

    %% Make sure we don't accidently overwrite a previous materials file
    if exist(materialsFile, 'file')
        fprintf('\nThe materials file\n%s\nalready exists!\n\n', materialsFile);
        str = input('Overwrite? (y/n): ', 's');
        if ~(strcmpi(str,'y') || strcmpi(str,'yes'))
            error('The materials file already exists for this subject!');
        end
    end
    
    
    %% Choose which materials to use for each run
    mainLanguage = inputs.mainLanguage;
    foreignLanguage = inputs.foreignLanguage;
    list = inputs.list;
    usingForeignLanguage = inputs.usingForeignLanguage;
    
    %For list 1, use odd intact materials and even degraded materials
    %For list 2, use even intact materials and odd degraded materials
    materialsPerCondition = 24;
    oddNums  = (1:2:materialsPerCondition);
    evenNums = (2:2:materialsPerCondition);
    
	if list == 1
        intactItemNums   = oddNums;
        degradedItemNums = evenNums;
    else
        intactItemNums   = evenNums;
        degradedItemNums = oddNums;
	end
    
    intactMaterials   = addStrToDouble(intactItemNums,   '%d_intact.wav');
    degradedMaterials = addStrToDouble(degradedItemNums, '%d_degraded.wav');
    foreignMaterials  = intactMaterials;
    
    %Randomize the order of the materials
    intactMaterials   = randomizeArray(intactMaterials);
    degradedMaterials = randomizeArray(degradedMaterials);
    foreignMaterials  = randomizeArray(foreignMaterials);
    
    %Prepend the appropriate path the the materials
    intactMaterials   = fullfile(mainLanguage,    'intact',   intactMaterials);
    degradedMaterials = fullfile(mainLanguage,    'degraded', degradedMaterials);
    foreignMaterials  = fullfile(foreignLanguage, 'intact',   foreignMaterials);
    
    %Split across 3 runs if including the foreign language condition
    %Split across 2 runs if not
    if usingForeignLanguage
        numRuns = 3;
    else
        numRuns = 2;
    end
    
	numLists = 2;
    materialsPerCondition = materialsPerCondition / numLists;
    
    if rem(materialsPerCondition, numRuns) ~= 0
        error('\nmaterialsPerCondition (%d) is not divisible by numRuns (%d)', ...
              materialsPerCondition, numRuns);
    end
    
    materialsPerRun = (materialsPerCondition / numRuns);
    for runIdx = 1:numRuns
        materialsEnd   = materialsPerRun * runIdx;
        materialsStart = materialsEnd - materialsPerRun + 1;
        materialsIndices = (materialsStart:materialsEnd);
        
        runField = sprintf('run%d', runIdx);
        subjMaterials.(runField).intact = intactMaterials(materialsIndices);
        subjMaterials.(runField).degraded = degradedMaterials(materialsIndices);
        
        if usingForeignLanguage
            subjMaterials.(runField).foreign = foreignMaterials(materialsIndices);
        end
    end
    
    %Save the subjMaterials structure so we can access it during run 2
    save(materialsFile, 'subjMaterials');
end


%% Sub-functions
function cellArray = addStrToDouble(arrayIn, format)
    cellArray = strtrim(cellstr(num2str(arrayIn',  format)));
end

function randomizedArray = randomizeArray(arrayIn)
    randomizedArray = arrayIn(randperm(length(arrayIn)));
end