function [pahandle, buffers] = setUpAudioFiles(materials, duration)
%setUpAudioFiles
%   Creates a buffer for each sound file
%
%   For each sound file, this script finds the channel with the maximum
%   intensity value. This channel is normalized and then copied numChannel
%   times. The other channels are disregarded.
    
    %% Makes sure fade duration isn't longer than audio duration
    assert(duration.fade <= duration.audio, ...
           'duration.fade must be less than or equal to duration.audio');
	
	audioDur = duration.audio;
    fadeDur  = duration.fade;
	
    
    %% Initialize the sound driver
    InitializePsychSound(1);
    PsychPortAudio('Close');
    
    sampleRate = 44100;
    numChannels = 2;
    
    pahandle = PsychPortAudio('Open', [], [], 0, sampleRate, numChannels);
    
    
    %% Create and fill one dynamic audio buffer for each sound file
    audioFiles = materials(~strcmp(materials, 'NA'));
    numAudioFiles = length(audioFiles);
    buffers = zeros(numAudioFiles,1);
    stimDir = fullfile(pwd, 'stimuli');
    
    for audioFileNum=1:numAudioFiles
         %% Read in the audio file
        audioFilename = audioFiles{audioFileNum};
        
        audioFile = fullfile(stimDir, audioFilename);
        try
            [audioSample, sampleRateIn] = audioread(audioFile);
        catch
            error(['Audio file not found:\n\t%s\n\nSee documentation at'...
                   ' the begining of the main script for instructions '...
                   'on how to set up the audio files.\n'], audioFile);
        end
        
        
        %% Normalize the audio file
        %Resample if necessary
        if sampleRateIn ~= sampleRate
            audioSample = resample(audioSample, sampleRate, sampleRateIn);
        end
        
        
        %Only keep the first audioDur seconds of the audio file
        sampleIndsToKeep = (1:audioDur*sampleRate);
        
        audioDurIn = length(audioSample)/sampleRate;
        if audioDurIn < audioDur
            warning(['The duration of audio file\n''%s''\nis %5.4f s, which '...
                     'is less than duration.audio (%5.4f s). The end of '...
                     'this audio clip will be padded with silence.\n'], ...
                     audioFile, audioDurIn, audioDur);
            
            numToPad = length(sampleIndsToKeep) - length(audioSample);
            audioSample = padarray(audioSample, numToPad, 'post');
        end
        
        audioSample = audioSample(sampleIndsToKeep, :);
        
        
        %Find the loudest channel in the audio sample (to use for playback)
        numChannelsIn = size(audioSample, 2);
        maxChannelIntensities = max(audioSample);
        maxIntensityIn = 0;
        loudestChannelNum = 0;
        
        for channelNum = 1:numChannelsIn
            maxChannelIntensity = maxChannelIntensities(channelNum);
            if maxChannelIntensity > maxIntensityIn
                maxIntensityIn = maxChannelIntensity;
                loudestChannelNum = channelNum;
            end
        end
        
        loudestChannel = audioSample(:, loudestChannelNum);
        
        
        %Scale the audio data to a maximum intensity (normalize volume)
        maxIntensity = 1;
        assert(maxIntensity <= 1, 'maxIntensity must be less than or equal to 1');
        intensityMultiplier = maxIntensity / maxIntensityIn;
        loudestChannel = loudestChannel .* intensityMultiplier;
        
        
        %Fade out the volume at the specified time
        fadeStart = (audioDur - fadeDur) * sampleRate;
        fadeEnd = length(loudestChannel);
        fadeInds = (fadeStart:fadeEnd);
        
        fadeScale = linspace(1,0,length(fadeInds));  %logspace(0,-1,length(fadeInds));
        
        loudestChannel(fadeInds) = loudestChannel(fadeInds) .* fadeScale';
        
        
        %Copy the loudest channel to each channel we want to play back
        audioSample = repmat(loudestChannel, 1, numChannels);
        
        
        %% Create and fill an audio buffer with the sound data
        waveData = audioSample';
        buffers(audioFileNum) = PsychPortAudio('CreateBuffer', [], waveData);
    end
end